/**
 * Forward accumulation of the sum of elements in a list
 */
public class SumVisitor implements IntListVisitor {
  public static final SumVisitor ONLY = new SumVisitor();
  
  private SumVisitor() {}
  
  public Object forEmptyIntList(EmptyIntList host) {
    return 0;
  }
  
  public Object forConsIntList(final ConsIntList host) {
    // Delegate to a helper visitor
    return host.getRest().accept(new IntListVisitor() {
      private int acc = host.getFirst();   // initialize the accumulator
      
      public Object forEmptyIntList(EmptyIntList hostHelper) {
        return acc;
      }
      
      public Object forConsIntList(final ConsIntList hostHelper) {
        acc += hostHelper.getFirst();
        return hostHelper.getRest().accept(this);
      }
      
    });
  }
}